--[[
Alcorn McBride V-Page 4
Q-Sys Plugin authored by B.Helman, Forward Thinking Designs
REV 1.4.1, April 13, 2018
]]

PluginInfo = 
{
  Name = "Alcorn McBride~V-Page 4",
  Version = "1.4.1",
  Id = "0c02812a-8c75-4867-af91-06c11ee59bcd",
  Description = "Controller componenet for an Alcorn McBride V-Page 4",
  ShowDebug = false
}

function GetColor( props )
	return { 147, 169, 213 }
end


function GetProperties()
  props = {
	{
		Name = "PA Station Number",
		Type = "integer",
		Min = 1,
		Max = 99,
		Value = 1,
	},
  {
		Name = "V-Page IP Address",
		Type = "string",
		Value = "192.168.0.250",
	},
  }
  return props
end


function GetPrettyName(props)
  return "V-Page 4" .. tostring( props["PA Station Number"].Value == "" and "" or "\r\nID: "..props["PA Station Number"].Value ) .. tostring( props["V-Page IP Address"].Value == "" and "" or "\r\n"..props["V-Page IP Address"].Value )
end


function RectifyProperties(props)
	
	return props
end


function GetControls(props)
  ctrls = {
  {
    Name = "PTTStatus",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "Link",
    ControlType = "Button",
    ButtonType = "Trigger",
    UserPin = true,
    PinStyle = "Input",
    Count = 1,
  },
  {
    Name = "ID",
    ControlType = "Button",
    ButtonType = "Trigger",
    UserPin = true,
    PinStyle = "Input",
    Count = 1,
  },
  {
    Name = "StatusLED",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "Status",
    ControlType = "Indicator",
    IndicatorType = "Status",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "StationName",
    ControlType = "Text",
    UserPin = true,
    PinStyle = "Both",
    Count = 1,
  },
  {
    Name = "StationPriority",
    ControlType = "Text",
    UserPin = true,
    PinStyle = "Both",
    Count = 1,
  },
  {
    Name = "VPID",
    ControlType = "Indicator",
    IndicatorType = "Text",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "VPIP",
    ControlType = "Indicator",
    IndicatorType = "Text",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "EmulationMode",
    ControlType = "Button",
    ButtonType = "Toggle",
    UserPin = true,
    PinStyle = "Output",
    Count = 1
  },
  {
    Name = "EmulationLED",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "OnlineLED",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "FeedbackLED",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  {
    Name = "Feedback",
    ControlType = "Indicator",
    IndicatorType = "Text",
    UserPin = true,
    PinStyle = "Output",
    Count = 1,
  },
  --[[
  {
    Name = "Refresh",
    ControlType = "Button",
    ButtonType = "Trigger",
    UserPin = false,
    Count = 1,
  },
  ]]
  {
    Name = "DefaultGroup",
    ControlType = "Text",
    UserPin = true,
    PinStyle = "Both",
    Count = 1,
  },
  {
    Name = "DefaultTimeout",
    ControlType = "Knob",
    ControlUnit = "Integer",
    Min = 0,
    Max = 600,
    Count = 1,
  },
  {
    Name = "ZoneSelection",
    ControlType = "Text",
    UserPin = true,
    PinStyle = "Both",
    Count = 4,
  },
  {
    Name = "ZoneActive",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Output",
    Count = 4,
  },
  {
    Name = "BAOffMode",
    ControlType = "Button",
    ButtonType = "Toggle",
    UserPin = false,
    Count = 4,
  },
  {
    Name = "BAToggleMode",
    ControlType = "Button",
    ButtonType = "Toggle",
    UserPin = false,
    Count = 4,
  },
  {
    Name = "BATriggerMode",
    ControlType = "Button",
    ButtonType = "Toggle",
    UserPin = false,
    Count = 4,
  },
  {
    Name = "BALED",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Input",
    Count = 4,
  },
  {
    Name = "BAState",
    ControlType = "Indicator",
    IndicatorType = "Led",
    UserPin = true,
    PinStyle = "Output",
    Count = 4,
  },
  }
  
  return ctrls
end

function GetControlLayout(props)
	layout = {}
	
  layout["PTTStatus"] = 
  {
    PrettyName = "Push-To-Talk Status",
		Style = "Led",
		Position = { 192, 19 },
		Size = { 16, 16 },
		Color = { 0, 255, 0 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		StrokeColor = { 0, 165, 79 },
		Margin = 3,
		Radius = 2,
		IsReadOnly = false,
  }
  
  layout["Link"] = 
	{
		PrettyName = "Link V-Page",
		Style = "Button",
		Position = { 314, 14 },
		Size = { 36, 24 },
		Color = { 0, 255, 255 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		ButtonStyle = "Trigger",
		HTextAlign = "Center",
		VTextAlign = "Center",
		Legend = "LINK",
		WordWrap = false,
		IsBold = false,
		TextFontSize = 10,
		Margin = 2,
		Radius = 2,
		IsReadOnly = false,
	}
  
  layout["ID"] = 
	{
		PrettyName = "Identify V-Page",
		Style = "Button",
		Position = { 374, 14 },
		Size = { 36, 24 },
		Color = { 248, 244, 0 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		ButtonStyle = "Trigger",
		HTextAlign = "Center",
		VTextAlign = "Center",
		Legend = "ID",
		WordWrap = false,
		IsBold = false,
		TextFontSize = 10,
		Margin = 2,
		Radius = 2,
		IsReadOnly = false,
	}
  
  layout["StatusLED"] = 
  {
    PrettyName = "Status LED",
		Style = "Led",
		Position = { 14, 50 },
		Size = { 16, 16 },
		Color = { 200, 200, 200 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		StrokeColor = { 0, 165, 79 },
    StrokeWidth = 0,
		Margin = 3,
		Radius = 2,
		IsReadOnly = false,
  }
  
  layout["Status"] = 
	{
		PrettyName = "Status",
		Style = "Text",
		Position = { 32, 42 },
		Size = { 378, 32 },
		--Color = { 0,0,0 },
		HTextAlign = "Center",
		VTextAlign = "Center",
		TextBoxStyle = "Normal",
		WordWrap = true,
		IsBold = false,
		TextFontSize = 10,
		Margin = 0,
		Radius = 0,
		IsReadOnly = false,
	}
  
  layout["StationName"] = 
	{
		PrettyName = "V-Page Name",
		Style = "Textbox",
		Position = { 114, 78 },
		Size = { 133, 16 },
		HTextAlign = "Center",
		VTextAlign = "Center",
		TextBoxStyle = "Normal",
		WordWrap = false,
		TextFontSize = 10,
		Margin = 0,
		Radius = 0,
		IsReadOnly = false,
	}
  
  layout["StationPriority"] = 
	{
		PrettyName = "Priority",
		Style = "ComboBox",
		Position = { 114, 98 },
		Size = { 133, 16 },
		HTextAlign = "Left",
		VTextAlign = "Center",
		WordWrap = false,
		TextFontSize = 10,
		IsReadOnly = false,
	}
  
  layout["VPID"] = 
	{
		PrettyName = "V-Page ID",
		Style = "Text",
		Position = { 314, 78 },
		Size = { 96, 16 },
		--Color = { 0,0,0 },
		HTextAlign = "Center",
		VTextAlign = "Center",
		TextBoxStyle = "Normal",
		WordWrap = false,
		IsBold = false,
		TextFontSize = 10,
		Margin = 0,
		Radius = 0,
		IsReadOnly = false,
	}
  
  layout["VPIP"] = 
	{
		PrettyName = "V-Page IP Address",
		Style = "Text",
		Position = { 314, 98 },
		Size = { 96, 16 },
		--Color = { 0,0,0 },
		HTextAlign = "Center",
		VTextAlign = "Center",
		TextBoxStyle = "Normal",
		WordWrap = false,
		IsBold = false,
		TextFontSize = 10,
		Margin = 0,
		Radius = 0,
		IsReadOnly = false,
	}
  
  layout["EmulationMode"] = 
	{
		PrettyName = "Enable Emulation Mode",
		Style = "Button",
		Position = { 120, 151 },
		Size = { 64, 16 },
		Color = { 0, 255, 0 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		ButtonStyle = "Toggle",
		HTextAlign = "Center",
		VTextAlign = "Center",
		Legend = "Enable",
		WordWrap = false,
		IsBold = false,
		TextFontSize = 10,
		Margin = 2,
		Radius = 2,
		IsReadOnly = false,
	}
  
  layout["EmulationLED"] = 
  {
    PrettyName = "Emulation Active",
		Style = "Led",
		Position = { 192, 151 },
		Size = { 16, 16 },
		Color = { 0, 255, 0 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		StrokeColor = { 0, 165, 79 },
		Margin = 3,
		Radius = 2,
		IsReadOnly = false,
  }
  
  layout["OnlineLED"] = 
  {
    PrettyName = "Connected to Core",
		Style = "Led",
		Position = { 350, 151 },
		Size = { 16, 16 },
		Color = { 0, 255, 0 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		StrokeColor = { 0, 165, 79 },
		Margin = 3,
		Radius = 2,
		IsReadOnly = false,
  }
  
  layout["FeedbackLED"] = 
  {
    PrettyName = "Paging Status LED",
		Style = "Led",
		Position = { 14, 203 },
		Size = { 16, 16 },
		Color = { 0, 255, 0 },
		UnlinkOffColor = false,
		--OffColor = { r, g, b },
		StrokeColor = { 0, 165, 79 },
    StrokeWidth = 0,
		Margin = 3,
		Radius = 2,
		IsReadOnly = false,
  }
  
  layout["Feedback"] = 
	{
		PrettyName = "Paging Status",
		Style = "Text",
		Position = { 32, 203 },
		Size = { 378, 16 },
		--Color = { 0,0,0 },
		HTextAlign = "Center",
		VTextAlign = "Center",
		TextBoxStyle = "Normal",
		WordWrap = false,
		IsBold = false,
		TextFontSize = 10,
		Margin = 0,
		Radius = 0,
		IsReadOnly = false,
	}
  --[[
  layout["Refresh"] =
  {
    PrettyName = "Reload Zone Assignment and Priority Choices",
    Style = "Button",
    ButtonStyle = "Trigger",
    Position = {120, 257},
    Size = {64, 16},
    Color = {0, 0, 255},
    UnlinkOffColor = false,
    Legend = "Refresh",
  }
  ]]
  layout["DefaultGroup"] = 
  {
    PrettyName = "Default Group Button",
    Style = "ComboBox",
    Position = { 249, 257 },
    Size = { 36, 16 },
    HTextAlign = "Center",
    VTextAlign = "Center",
  }
  
  layout["DefaultTimeout"] = 
  {
    PrettyName = "Default Group Selection Timeout (In Seconds)",
    Style = "Textbox",
    TextboxStyle = "Meter",
    Position = { 338, 257 },
    Size = { 36, 16 },
  }
  
  for i = 1, 4 do
    layout["ZoneSelection "..i] = 
    {
      PrettyName = "Button "..i.."~Zone Assignment",
      Style = "ComboBox",
      Position = { 43, ((i-1)*23+336) },
      Size = { 140, 16 },
      HTextAlign = "Left",
      VTextAlign = "Center",
      WordWrap = false,
      TextFontSize = 10,
      IsReadOnly = false,
    }
    
    layout["ZoneActive "..i] = 
    {
      PrettyName = "Button "..i.."~Zone Active LED",
      Style = "Led",
      Position = { 193, ((i-1)*23+336) },
      Size = { 16, 16 },
      Color = { 255, 0, 0 },
      UnlinkOffColor = false,
      --OffColor = { r, g, b },
      StrokeColor = { 188, 0, 29 },
      StrokeWidth = 1,
      Margin = 3,
      Radius = 2,
      IsReadOnly = false,
    }
    
    layout["BAOffMode "..i] = 
    {
      PrettyName = "Button "..i.."~Breakaway Off",
      Style = "Button",
      Position = { 234, ((i-1)*23+336) },
      Size = { 16, 16 },
      Color = { 150, 150, 150 },
      UnlinkOffColor = true,
      OffColor = { 255, 255, 255 },
      ButtonStyle = "Toggle",
      HTextAlign = "Center",
      VTextAlign = "Center",
      Legend = "",
      WordWrap = false,
      IsBold = false,
      TextFontSize = 10,
      Margin = 2,
      Radius = 8,
      StrokeColor = { 255, 255, 255 },
      StrokeWidth = 2,
      Padding = 2,
      IsReadOnly = false,
    }
    
    layout["BAToggleMode "..i] = 
    {
      PrettyName = "Button "..i.."~Breakaway Toggle",
      Style = "Button",
      Position = { 274, ((i-1)*23+336) },
      Size = { 16, 16 },
      Color = { 150, 150, 150 },
      UnlinkOffColor = true,
      OffColor = { 255, 255, 255 },
      ButtonStyle = "Toggle",
      HTextAlign = "Center",
      VTextAlign = "Center",
      Legend = "",
      WordWrap = false,
      IsBold = false,
      TextFontSize = 10,
      Margin = 2,
      Radius = 8,
      Padding = 2,
      StrokeColor = { 255, 255, 255 },
      StrokeWidth = 2,
      IsReadOnly = false,
    }
    
    layout["BATriggerMode "..i] = 
    {
      PrettyName = "Button "..i.."~Breakaway Trigger",
      Style = "Button",
      Position = { 315, ((i-1)*23+336) },
      Size = { 16, 16 },
      Color = { 150, 150, 150 },
      UnlinkOffColor = true,
      OffColor = { 255, 255, 255 },
      ButtonStyle = "Toggle",
      HTextAlign = "Center",
      VTextAlign = "Center",
      Legend = "",
      WordWrap = false,
      IsBold = false,
      TextFontSize = 10,
      Margin = 2,
      Radius = 8,
      Padding = 2,
      StrokeColor = { 255, 255, 255 },
      StrokeWidth = 2,
      IsReadOnly = false,
    }
    
    layout["BALED "..i] = 
    {
      PrettyName = "Button "..i.."~Breakaway Input LED",
      Style = "Led",
      Position = { 350, ((i-1)*23+336) },
      Size = { 16, 16 },
      Color = { 0, 255, 0 },
      UnlinkOffColor = false,
      --OffColor = { r, g, b },
      StrokeColor = { 0, 147, 55 },
      StrokeWidth = 1,
      Margin = 3,
      Radius = 2,
      IsReadOnly = false,
    }
    
    layout["BAState "..i] = 
    {
      PrettyName = "Button "..i.."~Breakaway State LED",
      Style = "Led",
      Position = { 386, ((i-1)*23+336) },
      Size = { 16, 16 },
      Color = { 255, 255, 0 },
      UnlinkOffColor = false,
      --OffColor = { r, g, b },
      StrokeColor = { 145, 232, 0 },
      StrokeWidth = 1,
      Margin = 3,
      Radius = 2,
      IsReadOnly = false,
    }
  end
  
	graphics = {  
  
  {
		Type = "GroupBox",  --Paging Setting
		Position = { 10, 10 },
		Size = { 404, 109 },
		Text = "V-Page Status",
		TextSize = 12,
		HTextAlign = "Left",
		IsBold = false,
		StrokeWidth = 1,
		CornerRadius = 8,
	},
  
  {
		Type = "Label",
		Position = { 92, 19 },
		Size = { 97, 16 },
		Text = "Push-To-Talk Status",
		TextSize = 10,
		HTextAlign = "Right",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
    Type = "Label",
    Position = { 247, 19 },
    Size = { 58, 16 },
    Text = "v"..PluginInfo.Version,
    HTextAlign = "Right",
    VTextAlign = "Center",
  },
  
  {
		Type = "Label",
		Position = { 14, 78 },
		Size = { 96, 16 },
		Text = "Station Name",
		TextSize = 12,
		HTextAlign = "Right",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 14, 98 },
		Size = { 96, 16 },
		Text = "Station Priority",
		TextSize = 12,
		HTextAlign = "Right",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 247, 78 },
		Size = { 63, 16 },
		Text = "V-Page ID",
		TextSize = 12,
		HTextAlign = "Right",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 247, 98 },
		Size = { 63, 16 },
		Text = "IP Address",
		TextSize = 12,
		HTextAlign = "Right",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "GroupBox",  --Network Link
		Position = { 10, 127 },
		Size = { 404, 45 },
		Text = "Network Link",
		TextSize = 12,
		HTextAlign = "Left",
		IsBold = false,
		StrokeWidth = 1,
		CornerRadius = 8,
	},
  
  {
		Type = "Label",
		Position = { 120,135 },
		Size = { 88, 16 },
		Text = "Emulation Mode",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 338,135 },
		Size = { 40, 16 },
		Text = "Online",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "GroupBox",  --Paging Status
		Position = { 10, 180 },
		Size = { 404, 45 },
		Text = "Paging Status",
		TextSize = 12,
		HTextAlign = "Left",
		IsBold = false,
		StrokeWidth = 1,
		CornerRadius = 8,
	},
  
  {
		Type = "GroupBox",  --Zone Button Configuration
		Position = { 10, 233 },
		Size = { 404, 197 },
		Text = "V-Page Button Configuration",
		TextSize = 12,
		HTextAlign = "Left",
		IsBold = false,
		StrokeWidth = 1,
		CornerRadius = 8,
	},
  --[[
  {
		Type = "Label",
		Position = { 33,257 },
		Size = { 87, 16 },
		Text = "Reload Settings",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  ]]
  {
		Type = "Label",
		Position = { 227,241 },
		Size = { 87, 16 },
		Text = "Default Button",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 314,241 },
		Size = { 88, 16 },
		Text = "Default Timeout",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Header",  --Paging Zones
		Position = { 43, 273 },
		Size = { 185, 26 },
		Text = "Paging Zones",
		TextSize = 12,
		HTextAlign = "Center",
		IsBold = false,
	},
  
  {
		Type = "Header",  --Break Away
		Position = { 238, 273 },
		Size = { 171, 26 },
		Text = "Breakaway",
		TextSize = 12,
		HTextAlign = "Center",
		IsBold = false,
	},
  
  {
		Type = "Header",  --Mode
		Position = { 234, 299 },
		Size = { 96, 16 },
		Text = "Mode",
		TextSize = 12,
		HTextAlign = "Center",
		IsBold = false,
	},
  
  {
		Type = "Label",
		Position = { 43, 315 },
		Size = { 140, 16 },
		Text = "Paging Zone Assignemnt",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 181, 315 },
		Size = { 39, 16 },
		Text = "Active",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 222, 315 },
		Size = { 40, 16 },
		Text = "Off",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 262, 315 },
		Size = { 40, 16 },
		Text = "Toggle",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 302, 315 },
		Size = { 40, 16 },
		Text = "Trigger",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 344, 315 },
		Size = { 30, 16 },
		Text = "LED",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
  {
		Type = "Label",
		Position = { 379, 315 },
		Size = { 30, 16 },
		Text = "State",
		TextSize = 12,
		HTextAlign = "Center",
		VTextAlign = "Center",
		IsBold = false,
		StrokeWidth = 0,
		CornerRadius = 0,
	},
  
	}
  
  for i = 1, 4 do
    table.insert( graphics, 
    {
      Type = "Label",
      Position = { 12, (i-1)*23+336 },
      Size = { 26, 16 },
      Text = tostring(i),
      TextSize = 12,
      HTextAlign = "Right",
      VTextAlign = "Center",
      IsBold = false,
      StrokeWidth = 0,
      CornerRadius = 0,
    })
    
    table.insert( graphics,
    {
      Type = "GroupBox",  --Paging Setting
      Position = { 234, (i-1)*23+336 },
      Size = { 16, 16 },
      Text = "",
      TextSize = 12,
      HTextAlign = "Left",
      IsBold = false,
      StrokeWidth = 1,
      StrokeColor = { 150, 150, 150 },
      CornerRadius = 8,
    })
    
    table.insert( graphics,
    {
      Type = "GroupBox",  --Paging Setting
      Position = { 274, (i-1)*23+336 },
      Size = { 16, 16 },
      Text = "",
      TextSize = 12,
      HTextAlign = "Left",
      IsBold = false,
      StrokeWidth = 1,
      StrokeColor = { 150, 150, 150 },
      CornerRadius = 8,
    })
    
    table.insert( graphics,
    {
      Type = "GroupBox",  --Paging Setting
      Position = { 315, (i-1)*23+336 },
      Size = { 16, 16 },
      Text = "",
      TextSize = 12,
      HTextAlign = "Left",
      IsBold = false,
      StrokeWidth = 1,
      StrokeColor = { 150, 150, 150 },
      CornerRadius = 8,
    })
  end
  
	return layout, graphics
end

function GetPins(props)
	local pins = {}
	
	return pins
end

function GetComponents(props)
	local comps = {}
	
	return comps
end

function GetWiring(props)
	local wiring = {}
	
	return wiring
end

if Controls then
	--Main Code
-----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------------
--[[  VARIABLE NAMING CONVENTIONS
CONSTANT_VARIABLES are defined in the head of the code and use CAPITAL_SNAKE_CASE
Assignable_Properties are declared in the head, but assigned by a function and use the Upper_Heart_Beat_Case
ObjectVariables are declared in the head, but may be manipulated by several functions and use the UpperCamelCase
holdingVariables are declared in the head, but are manipulated by several functions and use the lowerCamelCase
namedFunctions also use lowerCamelCase
localvariables are used only in functions and loops for temporary assignment and usage and use lowercase
]]--------------------------------------------------------------------------------------------------------------

--CONSTANT_DECLARATIONS
ZONE_COUNT = 4
VP_IP_ADDRESS = Properties["V-Page IP Address"].Value
PA_STATION = Properties["PA Station Number"].Value

CORE_PORT = 4000 + PA_STATION
AMI_PORT = 2638
VP_PORT = 5000
SELF_DISCOVERY_PORT = 555

HEARTBEAT_ACK = "TXXXXXX."
HEARTBEAT_REQ = "FE"
HEARTBEAT_TIMEOUT = 8
PTT_ON = "FF"
PTT_OFF = "00"
ZONE_CHANGE = "01"

STATUS_COLORS = { "#00FF00", "orange", "red", "gray", "darkred", "blue" }
ID_FLASH_INTERVAL = .250
BA_STATE_TRIGGER_TIMEOUT = 1

MAX_PAGE_DURATION = 0

INITIALIZATION_TIME = 10
UPDATE_LOCK_TIMEOUT = 0.01

REFRESH_RATE = 10


--Assignable_Properties
--Core_LAN_A = ""
--Core_LAN_B = ""
Core_LANs = {}
Active_Core_IP_Address = ""
Broadcast_IP_Address = "255.255.255.255"


--holdingVariables
--lanAActive = false
paRouterFound = false
vpOnline = true
updateLock = false

pageRequest = ""
paTags = {}
paPriorities = {}
paZones = {}

idFlashCount = 7
nextLAN = 1


--ObjectVariables
StationName = Controls.StationName
StationPriority = Controls.StationPriority
--DefaultZone = Controls.DefaultZone
ZoneSelection = Controls.ZoneSelection
ZoneActive = Controls.ZoneActive
BAEnable = Controls.BAEnable
BALED = Controls.BALED
BAState = Controls.BAState
BAOffMode = Controls.BAOffMode
BAToggleMode = Controls.BAToggleMode
BATriggerMode = Controls.BATriggerMode
ID = Controls.ID
Link = Controls.Link
Status = Controls.Status
StatusLED = Controls.StatusLED
OnlineLED = Controls.OnlineLED
PTTStatus = Controls.PTTStatus
VPID = Controls.VPID
VPIP = Controls.VPIP
EmulationLED = Controls.EmulationLED
EmulationMode = Controls.EmulationMode
Feedback = Controls.Feedback
FeedbackLED = Controls.FeedbackLED
DefaultGroup = Controls.DefaultGroup
DefaultTimeout = Controls.DefaultTimeout
--Refresh = Controls.Refresh

HeartbeatTimer = Timer.New()
IDTimer = Timer.New()
StateTimers = {}
DefaultTimer = Timer.New()
InitializationTimer = Timer.New()
UpdateLockTimer = Timer.New()
RefreshTimer = Timer.New()

UDP = UdpSocket.New()


-------------------------------------------------------------------------------------------------------------------------------------Paging Functions

function updatePagingStatus( value, message )

  if not message then
    if not paRouterFound then
      Status.Value = 2
      Status.String = "No PA Router present."
      StatusLED.Boolean = true
      StatusLED.Color = STATUS_COLORS[3]
    elseif not vpOnline then
      Status.Value = 2
      Status.String = "V-Page not connected."
      StatusLED.Boolean = true
      StatusLED.Color = STATUS_COLORS[3]
    else
      Status.Value = 0
      StatusLED.Boolean = true
      StatusLED.Color = STATUS_COLORS[1]
    end
  else
    Status.Value = value
    Status.String = message
    StatusLED.Boolean = value ~= 3
    StatusLED.Color = STATUS_COLORS[value + 1]
  end
end


function clearBAState( zone )

  BAState[zone].Value = 0

end


function updateVPIndicators()

  --first determine which zones/tags are mapped to this VP
  local stateflags = {}
  for i,v in ipairs(ZoneSelection) do
    --create a pairing of the labels we're looking for and whether they are active or not
    table.insert( stateflags, { Label = v.String, Flag = false } )
  end
  
  --compare the names of all the zones and tags to each button
  for i,v in ipairs(stateflags) do
    --check zones first
    for j,w in ipairs(paZones) do
      if w.Name == v.Label then
        v.Flag = w.Active
      end
    end
    
    --check tags second
    for j,w in ipairs(paTags) do
      if w.Name == v.Label then
        --determine if any of the tag members are active
        local flag = false
        for k,x in ipairs(w.Zones) do
          if x.Active then
            flag = true
          end
        end
        --at least one zone member is active so the entire tag is set to active
        v.Flag = flag
      end
    end
  end
  
  --send update messages to the VP based on the Flag property of stateflags
  for i,v in ipairs(stateflags) do
    sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, tostring( v.Flag==true and "01" or "02" ) ) )
    if PTTStatus.Boolean then
      ZoneActive[i].Boolean = v.Flag
    else
      ZoneActive[i].Boolean = false
    end
  end
  
end


function getZonesFromName( name )

  for i,v in ipairs(paZones) do
    if v.Name == name then
      return {i}
    end
  end

  return {}
  
end


function getPriorityFromName( name )

  for i,v in ipairs(paPriorities) do
    if v.Name == name then
      return (i-1) --priorities sent via the PARouter API are 0 based, for some reason
    end
  end
  
  return nil

end


function getPriorityColorFromName( name )

  for i,v in ipairs(paPriorities) do
    if v.Name == name then
      return v.Color
    end
  end
  
  return "#FFFFFF"
  
end


function preparePagingRequest( zonebutton )

  local zonesselected = getZonesFromName( ZoneSelection[zonebutton].String )
  local params = {
		Mode = "live",
		Zones = zonesselected,
		Priority = getPriorityFromName( StationPriority.String ),
		Station = PA_STATION,
    MaxPageTime = MAX_PAGE_DURATION,
	}

  if string.sub( ZoneSelection[zonebutton].String, 1, 3 ) == "Tag" then
    params["ZoneTags"] = { string.sub( ZoneSelection[zonebutton].String, 5 ) }
  end
  
	status, err = pcall( function () 
		pageRequest = PARouter.Submit( params )
	end )
	if not status then
    if string.find( err, "out of range" ) then
      Feedback.String = "PA Station Number is higher than available PA Router inputs."
      FeedbackLED.Value = 0
      pageRequest = ""
    end
    if string.find( err, "illegal tag" ) then
      Feedback.String = "Paging tag not found. Please reassign zone."
      FeedbackLED.Value = 0
      pageReqeust = ""
    end
	end
  
end


PARouter.EventHandler = function( rq, parstate )

  Feedback.String = parstate.Message
  if parstate.SubState == "immediate" or parstate.State == "input" then
    FeedbackLED.Value = 1
  else
    FeedbackLED.Value = 0
  end
  
  if parstate.State == "done" then
    pageRequest = ""
  end

end


------------------------------------------------------------------------------------------------------------------------------------------------------VPAGE SETUP
function identifyVP()

  --send the command to each local broadcast address to try each NIC, including AUX NICs
  for _, nic in ipairs( Core_LANs ) do
    UDP:Send( nic.Broadcast, VP_PORT, string.format( "T%02dXXXX.", PA_STATION ) )
  end
  
end


function rebootVP()

  --send the command to each local broadcast address to try each NIC, including AUX NICs
  for _, nic in ipairs( Core_LANs ) do
    UDP:Send( nic.Broadcast, VP_PORT, VP_IP_ADDRESS.."XX\r" )
  end
  
end


function setupVP()

  --set VPAGE Q-Sys target IP
  UDP:Send( VP_IP_ADDRESS, AMI_PORT, Active_Core_IP_Address.."QIP\r" )
  --set VPAGE Q-Sys target port
  UDP:Send( VP_IP_ADDRESS, AMI_PORT, CORE_PORT.."QPR\r" )
  --set VPAGE ID number
  UDP:Send( VP_IP_ADDRESS, AMI_PORT, PA_STATION.."ID\r" )
  --disable default group timeout as it's handled by the plugin
  UDP:Send( VP_IP_ADDRESS, AMI_PORT, "0DGTO\r" )
  --reset the vponline flag to ensure all the breakaway and default settings are pushed
  --after the vpage connects
  vpOnline = false
  
end


function linkVP()
  
  --send the link packet to each local broadcast address to try each NIC, including AUX NICs
  for _, nic in ipairs( Core_LANs ) do
    UDP:Send( nic.Broadcast, VP_PORT, VP_IP_ADDRESS .. "," .. Active_Core_IP_Address .. "," .. CORE_PORT .. ",1LI\r" )
  end
  vpOnline = false
  
end


function sendMessage( message )

  if System.IsEmulating and not EmulationMode.Boolean then return end
  if not vpOnline then return end
  
  UDP:Send( VP_IP_ADDRESS, VP_PORT, message )

end


------------------------------------------------------------------------------------------------------------------------------------------------------PLUGIN SETUP
function getPagingConfig()

  local filepath = "" 
  if System.IsEmulating then
  	filepath = string.match(package.path, "(%a:\\Users.+)\\settings").."\\Settings\\paging_config.xml"
  else
  	filepath = "/usr/qsc/www/designs/current_design/settings/paging_config.xml"
  end

  local xmldoc = {}
  for line in io.lines(filepath) do
    if not line:find("<%?xml") then
     table.insert( xmldoc, line )
    end
  end
  local pconfig = xml.eval( table.concat( xmldoc ) )
  
  paTags = {}
  paPriorities = {}

  --parse out tags and priorities first
  for node = 1, #pconfig do
    if pconfig[node][0] == "Tags" then
      for tag = 1, #pconfig[node] do
        table.insert( paTags, { Name = "Tag:"..pconfig[node][tag].Name, Zones = {} } )
      end
    elseif pconfig[node][0] == "Priorities" then
      for priority = 1, #pconfig[node] do
        table.insert( paPriorities, { Name = pconfig[node][priority].Name, Color = pconfig[node][priority].Color } )
      end
    end
  end

  paZones = {}

  --then parse out zones.  The tag table must be built first.
  for node = 1, #pconfig do
    if pconfig[node][0] == "Zones" then
      for zone = 1, #pconfig[node] do
        paZones[tonumber(pconfig[node][zone].Index)] = { Name = pconfig[node][zone].Index.."-"..pconfig[node][zone].Name, Tags = { }, Active = false }
        for tag = 1, #pconfig[node][zone][1] do
          table.insert( paZones[zone].Tags, "Tag:"..pconfig[node][zone][1][tag].Name )
        end
      end
    end
  end

  --now transform the zones with tags to tags with zones.  This will make it easier to lookup zones when paging to a tag.
  for i,v in ipairs(paZones) do
    for j,w in ipairs(v.Tags) do
      for k,x in ipairs(paTags) do
        if w == x.Name then
          table.insert( x.Zones, { Index = i, Active = false } )
        end
      end
    end
  end

end


function updatePagingChoices()
  
  getPagingConfig()
  
  --load the paging priorities into the zone assignment combo boxes
  local t = {}
  for i,v in ipairs(paPriorities) do
    t[i] = v.Name
  end
  StationPriority.Choices = t
  if not getPriorityFromName( StationPriority.String ) then
    StationPriority.String = ""
  else
    StationPriority.Color = getPriorityColorFromName( StationPriority.String )
  end
  
  --load the paging zones and tags into the zone selection combo boxes
  t = {}
  table.insert( t, "<None>" )
  for i,v in ipairs(paZones) do
    table.insert( t, v.Name) 
  end
  for i,v in ipairs(paTags) do
    table.insert( t, v.Name )
  end
  for i,v in ipairs(ZoneSelection) do
    v.Choices = t
  end
  t = nil
  
end


function getCoreIPAddresses()
  
  --load the "kitchensink" XML file from the core's web root
  local filepath = "/usr/qsc/www/status.xml"
  local file = io.open( filepath )
  local xmldoc = file:read( "*a" )
  file:close()
  
  local statusinfo = xml.eval( xmldoc )
  local interfaces = {}
  --search through the status.xml file and look for network_interfaces
  for nodes=1, #statusinfo do
    if statusinfo[nodes][0] == "network_interfaces" then
      for interface=1, #statusinfo[nodes] do
        interfaces[interface] = {}
        for intnode = 1, #statusinfo[nodes][interface] do
          --translate all the xml elements in each "interface" object to key/value pairs in the "interfaces" table
          interfaces[interface][statusinfo[nodes][interface][intnode][0]] = statusinfo[nodes][interface][intnode][1]
        end
      end
    end
  end
  --translate the fields of interest into a prettier table
  local NICs = {}
  for i,v in ipairs(interfaces) do
    table.insert( NICs, 
    {
      Name = interfaces[i].name,
      Device = interfaces[i].linux_device_name,
      Address = interfaces[i].address,
      Mask = interfaces[i].mask,
      MAC = interfaces[i].mac_address,
      Broadcast = interfaces[i].broadcast,
      Gateway = interfaces[i].gateway,
      Chassis = interfaces[i].chassis,
    })
  end
  
  return NICs
  
end


function findLocalIPAddress()

  local xUDP = UdpSocket.New()
  local function updateLocalIPAddress( udp, packet )
    --set the active ip to the received packet sender address, which is our local computer running the emulation
    Active_Core_IP_Address = packet.Address
    xUDP:Close()
    setupVP()
  end
  xUDP.EventHandler = updateLocalIPAddress
  xUDP:Open( "", SELF_DISCOVERY_PORT )
  --sending a packet to a core's hostname will reveal its IP address
  pcall( function()
    xUDP:Send( "255.255.255.255", SELF_DISCOVERY_PORT, "EMULATION MODE" )
  end )
  
end


function setActiveIPAddress()

  EmulationLED.Value = 0
  OnlineLED.Value = 0
  vpOnline = false
  updatePagingStatus()
  if System.IsEmulating and EmulationMode.Boolean then
    findLocalIPAddress() --call setupVP itself because of asynchronous behavior
  elseif not System.IsEmulating then
    Core_LANs = getCoreIPAddresses()
    
    if nextLAN > #Core_LANs then
      nextLAN = 1
    end
    Active_Core_IP_Address = Core_LANs[nextLAN].Address
    nextLAN = nextLAN + 1
    
    setupVP() --we need to call this manually when running on a Core
  end
  
end


--------------------------------------------------------------------------------------------------------------------------------GUI EVENT HANDLERS
ID.EventHandler = function()

  identifyVP()
  IDTimer:Start(ID_FLASH_INTERVAL)
  
end


for i,v in ipairs(BAOffMode) do

  v.EventHandler = function(ctl)
    if ctl.Boolean then
      ZoneSelection[i].IsDisabled = false
      if BAToggleMode[i].Boolean then
        BALED[i].Boolean = false
        BAState[i].Boolean = false
      end
      BAToggleMode[i].Boolean = false
      BATriggerMode[i].Boolean = false
      if ZoneSelection[i].String ~= "<None>" then
        sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "05" ) )
      else
        sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "04" ) )
      end
    else
      ctl.Boolean = true
    end
  end

end


for i,v in ipairs(BAToggleMode) do

  v.EventHandler = function(ctl)
    if ctl.Boolean then
      ZoneSelection[i].IsDisabled = true
      BAOffMode[i].Boolean = false
      BATriggerMode[i].Boolean = false
      sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, tostring( BAState[i].Boolean==true and "03" or "04" ) ) )
    else
      ctl.Boolean = true
    end
  end
  
end


for i,v in ipairs(BATriggerMode) do

  v.EventHandler = function(ctl)
    if ctl.Boolean then
      ZoneSelection[i].IsDisabled = true
      BAToggleMode[i].Boolean = false
      BAOffMode[i].Boolean = false
      sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, tostring( BALED[i].Boolean==true and "03" or "04" ) ) )
      BAState[i].Value = 0
    else
      ctl.Boolean = true
    end
  end
  
end


Link.EventHandler = linkVP


StationPriority.EventHandler = function()

  StationPriority.Color = getPriorityColorFromName( StationPriority.String )

end


------------------------------------------------------------------------------------------------------------------------------------------TIMERS
HeartbeatTimer.EventHandler = function()

  --if messages have not been received from the VPage, it could be one of two issues:
  --1. The VPage is simply not online
  --2. The VPage is connected to LAN B instead of the assumed LAN A.
  --In the latter case, we need to try switching to using the other NIC for communication, but only if we are running on a Core
  setActiveIPAddress()
  
end


IDTimer.EventHandler = function()

  if idFlashCount == 0 then
    ID.Color = "#7A7400"
    idFlashCount = 7
    IDTimer:Stop()
  end
  if idFlashCount % 2 == 0 then
    ID.Color = "#FAF400"
  else
    ID.Color = "#7A7400"
  end
  idFlashCount = idFlashCount - 1
  
end


DefaultTimer.EventHandler = function()

  --[[ This section is now deprecated as the default group timer is handled by the V-Page firmware
  DefaultTimer:Stop()
  UDP:Send( VP_IP_ADDRESS, AMI_PORT, DefaultGroup.String.."DG\r" )
  preparePagingRequest( tonumber( DefaultGroup.String ) )
  ]]
  
end

-------------------------------------------------------------------------------------------------------------------------Comm Handling Functions
UDP.EventHandler = function(udp, packet)
  
  if packet.Address ~= VP_IP_ADDRESS then return end
  
  local data = packet.Data
  
  if System.IsEmulating and not EmulationMode.Boolean then return end
  
  if string.sub( data, 1, 1 ) ~= "T" and string.sub( data, #data, #data ) ~= "." then return end
  
  local VP = tonumber( string.sub( data,2,3 ) )
  local zone = tonumber( string.sub( data,4,5 ) )
  local message = string.sub( data,6,7 )
  
  --set up the vpage with defaults
  if not vpOnline then
    identifyVP()
    vpOnline = true
    
    --breakaway settings
    for i = 1, ZONE_COUNT do
      if BAToggleMode[i].Boolean then
        sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, tostring( BAState[i].Boolean==true and "03" or "04" ) ) )
      elseif BATriggerMode[i].Boolean then
        BAState[i].Value = 0
        sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, tostring( BALED[i].Boolean==true and "03" or "04" ) ) )
      else
        if ZoneSelection[i].String ~= "<None>" then
          sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "05" ) )
        else
          sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "04" ) )
        end
      end
    end
    
    --default group
    UDP:Send( VP_IP_ADDRESS, AMI_PORT, DefaultGroup.String.."DG\r" )
    preparePagingRequest( tonumber( DefaultGroup.String ) )
    
    updatePagingStatus()
  end
  
  --update the gui to show that we are communicating with the vpage
  if System.IsEmulating then
    EmulationLED.Value = 1
    OnlineLED.Value = 0
  else
    EmulationLED.Value = 0
    OnlineLED.Value = 1
  end
  
  --heartbeat request
  if message == HEARTBEAT_REQ then
    sendMessage( HEARTBEAT_ACK )
    HeartbeatTimer:Stop()
    HeartbeatTimer:Start( HEARTBEAT_TIMEOUT )
    
    updatePagingStatus()
    return
  end
  
  --PTT key on
  if message == PTT_ON then
    --DefaultTimer:Stop()
    PTTStatus.Value = 1
    if pageRequest ~= "" then
      pageRequest:Start()
    else
      Feedback.String = "Not ready."
      FeedbackLED.Value = 0
      preparePagingRequest( zone )
      pcall( function() pageRequest:Start() end )
    end
  end
  
  --PTT key off
  if message == PTT_OFF then
    PTTStatus.Value = 0
    pcall( function() pageRequest:Stop() end )
    --[[if DefaultTimeout.Value ~= 0 then
      DefaultTimer:Start( DefaultTimeout.Value )
    end]]
  end
  
  --zone selection change
  if message == ZONE_CHANGE then
    if not BAOffMode[zone].Boolean then
      if BAToggleMode[zone].Boolean then
        if BAState[zone].Boolean then
          BAState[zone].Boolean = false
          BALED[zone].Boolean = false
          sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, zone, "04" ) )
        else
          BAState[zone].Boolean = true
          BALED[zone].Boolean = true
          sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, zone, "03" ) )
        end
      elseif BATriggerMode[zone].Boolean then
        BAState[zone].Value = 1
        StateTimers[zone]:Start(BA_STATE_TRIGGER_TIMEOUT)
      end
    else
      if ZoneSelection[zone].String ~= "<None>" then
        preparePagingRequest( zone )
      end
    end
    --[[
    --restart the selection timeout period
    DefaultTimer:Stop()
    if DefaultTimeout.Value ~= 0 then
      DefaultTimer:Start( DefaultTimeout.Value )
    end]]
  end
  
end


-------------------------------------------------------------------------------------------------------------------------------------------INITIALIZE
function Initialize()

  InitializationTimer:Stop()
  
  VPID.String = PA_STATION
  VPIP.String = VP_IP_ADDRESS
  
  Status.Value = 3
  Status.String = "Waiting for VPage to connect."
  StatusLED.Boolean = false
  StatusLED.Color = STATUS_COLORS[4]
  
  UDP:Open( "", CORE_PORT )
  setActiveIPAddress()
  HeartbeatTimer:Start( HEARTBEAT_TIMEOUT )
  
  --attempt to tie an eventhandler to PAZoneStatus (the PA Router Status API object)
  --it's wrapped in a pcall because if there is no PA Router then it will throw an error
  getPagingConfig()
  
  local ok = pcall( function()
  paRouterFound = true
  PAZoneStatus.EventHandler = function( znTable )

    --update that active status in the zone and tag lists
    paZones[znTable.Zone].Active = znTable.Active
    
    if #paTags > 0 then
      for i,v in ipairs(paTags) do
        for j,w in ipairs(v.Zones) do
          if w.Index == znTable.Zone then
            w.Active = znTable.Active
          end
        end
      end
    end

    --update the paging station according to the activity states
    --this event handler is called each time the PA router notifies of a zone change.  Paging to multiple zones results in multiple, rapid calls.
    --this floods the paging stations on the network with duplicated/unnecessary packets.
    --To prevent this, we have a lockout timer that waits for all the ruckus to finish for 10ms.  Then we send a final update.
    --If more updates come after that, the lockout will be reengaged and wait another 10ms to fire again.
    if not updateLock then
      UpdateLockTimer:Start(UPDATE_LOCK_TIMEOUT)
      updateLock = true
    end
    --updateVPIndicators()
  end
  end )
  
  if not ok then
    paRouterFound = false
  else
    paRouterFound = true
  end
  
  RefreshTimer.EventHandler = function()
    updatePagingChoices()
    Core_LANs = getCoreIPAddresses()
  end
  updatePagingChoices()
  RefreshTimer:Start( REFRESH_RATE )
  
  --initialize states and gui elements
  for i = 1, ZONE_COUNT do
    if BAOffMode[i].Value == 0 and BAToggleMode[i].Value == 0 and BATriggerMode[i].Value == 0 then
      BAOffMode[i].Value = 1
    end
    
    if ZoneSelection[i].String == "" then
      ZoneSelection[i].String = "<None>"
    end
    
    if BAOffMode[i].Value == 0 then
      ZoneSelection[i].IsDisabled = true
    else
      ZoneSelection[i].IsDisabled = false
    end
    
    BALED[i].EventHandler = function()
      if BATriggerMode[i].Boolean then
        sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, tostring( BALED[i].Boolean==true and "03" or "04" ) ) )
      end
    end
    
    ZoneSelection[i].EventHandler = function(ctl)
      if ctl.String == "<None>" and BAOffMode[i].Boolean then
        sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "04" ) )
      elseif BAOffMode[i].Boolean then
        sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "05" ) )
      end
    end
    
    if ZoneSelection[i].String == "<None>" and BAOffMode[i].Boolean then
      sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "04" ) )
    elseif BAOffMode[i].Boolean then
      sendMessage( string.format( "T%02d%02d%02d.", PA_STATION, i, "05" ) )
    end
  end
  
  local groups = {}
  --initialize the breakaway led timers and default group choices
  for i = 1, ZONE_COUNT do
    StateTimers[i] = Timer.New()
    StateTimers[i].EventHandler = function()
      clearBAState( i )
    end
    table.insert( groups, tostring(i) )
  end
  DefaultGroup.Choices = groups
  if DefaultGroup.String == "" then DefaultGroup.String = "1" end
  DefaultGroup.EventHandler = function()
    --[[if DefaultTimeout.Value ~= 0 then
      DefaultTimer:Start( DefaultTimeout.Value )
    end]]
    UDP:Send( VP_IP_ADDRESS, AMI_PORT, DefaultGroup.String.."DG\r" )
  end
  
  DefaultTimeout.EventHandler = function()
    UDP:Send( VP_IP_ADDRESS, AMI_PORT, DefaultTimeout.Value.."DGTO\r" )
  end
  
  UpdateLockTimer.EventHandler = function()
    UpdateLockTimer:Stop()
    updateLock = false
    updateVPIndicators()
  end

  updatePagingStatus()
end


InitializationTimer.EventHandler = Initialize
InitializationTimer:Start(INITIALIZATION_TIME)
Status.Value = 5
Status.String = ""
StatusLED.Boolean = false
StatusLED.Color = STATUS_COLORS[6]
EmulationLED.Value = 0
OnlineLED.Value = 0
EmulationMode.Boolean = false

end